"""
Interface for MDEngine objects. New MDEngines can be implemented by constructing a new class that inherits from MDEngine
and implements its abstract methods.
"""

import abc
import os
import pytraj
import mdtraj
import mdtraj.formats

class MDEngine(abc.ABC):
    """
    Abstract base class for molecular dynamics engines.

    Implements methods for all of the engine-specific tasks that ATESA might need.

    """

    @abc.abstractmethod
    def get_frame(self, trajectory, frame, settings):
        """
        Return a new file containing just the frame'th frame of a trajectory in Amber .rst7 format

        Parameters
        ----------
        trajectory : str
            Name of trajectory file to obtain last frame from
        frame : int
            Index of frame to return; 1-indexed, -1 gives last frame, 0 is invalid
        settings : argparse.Namespace
            Settings namespace object

        Returns
        -------
        last_frame : str
            Name of .rst7 format coordinate file corresponding to desired frame of trajectory, if it exists; an empty
            string otherwise

        """
        pass

    @abc.abstractmethod
    def write_find_ts_restraint(self, basin, inp_file):
        """
        Return the input file for a restrained simulation that performs barrier crossing in find_ts

        Parameters
        ----------
        basin : tuple
            Either settings.commit_fwd or settings.commit_bwd, defining the basin to restrain towards
        inp_file : str
            Name of original input file without restraint

        Returns
        -------
        input_file : str
            Name of the simulation input file

        """
        pass


class AdaptAmber(MDEngine):
    """
    Adapter class for Amber MDEngine.

    """

    def get_frame(self, trajectory, frame, settings):
        new_restart_name = trajectory + '_frame_' + str(frame) + '.rst7'
        if not os.path.exists(trajectory):
            return ''   # since it's possible to call this before the trajectory file has been initialized
        if frame >= 1:
            shift_frame = frame - 1     # because write_traj is 0-indexed but get_frame is 1-indexed
        elif frame == -1:
            shift_frame = -1
        else:
            raise IndexError('invalid frame index for get_frame: ' + str(frame) + ' (must be >= 1, or exactly -1)')

        # Check for non-zero trajectory length
        # try:  # todo: remove, deprecated
        #     with pytraj.utils.context.capture_stdout():     # suppress C++ errors if this file is empty
        #         traj = pytraj.iterload(trajectory, settings.topology)
        #     if traj.n_frames == 0:
        #         del traj
        #         return ''
        # except ValueError:      # sometimes this is the result of trying to load a trajectory too early
        #     return ''
        traj = mdtraj.load(trajectory, top=settings.topology)
        if traj.n_frames == 0:
            return ''

        try:
            traj[shift_frame].save_amberrst7(new_restart_name, force_overwrite=True)
            # pytraj.write_traj(new_restart_name, traj, format='rst7', frame_indices=[shift_frame], options='multi', overwrite=True, velocity=True)
        except IndexError:
            raise IndexError('frame index ' + str(frame) + ' is out of range for trajectory: ' + trajectory)
        # except ValueError:  # pytraj raises a ValueError if frame index is out of range # todo: remove, deprecated
        #     raise IndexError('frame index ' + str(frame) + ' is out of range for trajectory: ' + trajectory)
        # except AssertionError:  # sometimes there's an assertion error when shift_frame = -1; cause unknown, but this fixes it
        #     if shift_frame == -1:
        #         shift_frame = traj.n_frames - 1
        #     try:
        #         pytraj.write_traj(new_restart_name, traj, format='rst7', frame_indices=[shift_frame], options='multi',
        #                           overwrite=True, velocity=True)
        #     except ValueError:  # pytraj raises a ValueError if frame index is out of range
        #         raise IndexError('frame index ' + str(frame) + ' is out of range for trajectory: ' + trajectory)
        # try:
        #     os.rename(new_restart_name + '.1', new_restart_name)
        # except OSError:
        #     if not os.path.exists(new_restart_name):
        #         raise OSError('expected pytraj to write either ' + new_restart_name + ' or ' + new_restart_name + '.1, '
        #                       'but found neither.\nThe most likely explanation is that the ATESA process is unable to '
        #                       'write to the working directory (' + settings.working_directory + '). You may have run '
        #                       'out of storage space.')
        if not os.path.exists(new_restart_name):
            raise OSError('expected mdtraj to write ' + new_restart_name + ' but it was not found.\nThe most likely '
                          'explanation is that the ATESA process is unable to write to the working directory (' +
                          settings.working_directory + '). You may have run out of storage space.')

        return new_restart_name

    def write_find_ts_restraint(self, basin, inp_file):
        # Writing an amber-style restraint file from scratch
        open('find_ts_restraints.disang', 'w').write('')  # initialize file
        with open('find_ts_restraints.disang', 'a') as f:
            f.write('DISANG restraint file produced by ATESA with find_ts > 0 to produce transition state guesses\n')
            for def_index in range(len(basin[0])):
                extra = 0  # additional distance to add to basin definition to push *into* basin rather than to its edge
                if basin[3][def_index] == 'lt':
                    extra = -0.1 * basin[2][def_index]  # minus 10% # todo: this doesn't work for negative angles/dihedrals (moves them towards zero instead of "more negative"), which is fine for now since angles and dihedrals are not yet supported
                elif basin[3][def_index] == 'gt':
                    extra = 0.1 * basin[2][def_index]   # plus 10%
                else:
                    raise RuntimeError('entries in the last list in commitment definitions (commit_fwd and commit_bwd) '
                                       'must be either \'lt\' (less than) or \'gt\' (greater than)')

                f.write(' &rst\n')
                f.write('  iat=' + str(basin[0][def_index]) + ',' + str(basin[1][def_index]) + ',\n')
                f.write('  r1=0, r2=' + str(basin[2][def_index] + extra) + \
                        ', r3=' + str(basin[2][def_index] + extra) + \
                        ', r4=' + str(basin[2][def_index] + extra + 2) + ',\n')
                f.write('  rk2=500, rk3=500,\n')
                f.write(' &end\n')

        return inp_file     # unmodified, as find_ts_amber already includes the reference to find_ts_restraints.disang



"""
This portion of the program is responsible for handling update of the results, checking global termination criteria, and
implementing the calls to JobType methods to control the value of the thread.coordinates attribute for the next step.
"""

from atesa import factory

def interpret(thread, allthreads, running, settings):
    """
    The main function of interpret.py. Makes calls to JobType methods to update results, check termination criteria, and
    update thread.coordinates

    Parameters
    ----------
    thread : Thread
        The Thread object on which to act
    allthreads : list
        The list of all extant Thread objects
    running : list
        The list of all currently running Thread objects
    settings : argparse.Namespace
        Settings namespace object

    Returns
    -------
    termination: bool
        True if a global termination criterion has been met; False otherwise

    """

    jobtype = factory.jobtype_factory(settings.job_type)
    termination = False

    if jobtype.check_for_successful_step(thread, settings):                     # ensure this step did not crash/fail
        jobtype.update_results(thread, allthreads, settings)                    # update results as needed
        termination = jobtype.check_termination(thread, allthreads, settings)   # check global termination criteria
    running = jobtype.algorithm(thread, allthreads, running, settings)          # set thread parameters for next step

    if settings.DEBUG:
        termination = True
        running = [thread]

    return termination, running



#!/usr/bin/env python3

"""
main.py
Aimless Transition Ensemble Sampling and Analysis (ATESA)

This script handles the primary loop of building and submitting jobs in independent Threads, using the methods thereof 
to execute various interfaced/abstracted commands.
"""

import sys
import os
import shutil
import pickle
import pytraj
import copy
import time
import glob
import psutil
import warnings
import itertools
from atesa import configure
from atesa import factory
from atesa import process
from atesa import interpret
from atesa import utilities
from atesa import information_error
from atesa import resampling
from multiprocess import Pool, Manager, get_context

class Thread(object):
    """
    Object representing a series of simulations and containing the relevant information to define its current state.

    Threads represent the level on which ATESA is parallelized. This flexible object is used for every type of job
    performed by ATESA.

    Parameters
    ----------
    settings : argparse.Namespace
        Settings namespace object

    Returns
    -------
    None

    """

    def __init__(self):
        self.topology = ''              # filename containing topology file
        self.jobids = []                # list of jobids associated with the present step of this thread
        self.terminated = False         # boolean indicating whether the thread has reached a termination criterion
        self.current_type = []          # list of job types for the present step of this thread
        self.current_name = []          # list of job names corresponding to the job types
        self.current_results = []       # results of each job, if applicable
        self.name = ''                  # name of current step
        self.suffix = 0                 # index of current step
        self.total_moves = 0            # running total of "moves" attributable to this thread
        self.accept_moves = 0           # running total of "accepted" "moves", as defined by JobType.update_results
        self.status = 'fresh thread'    # tag for current status of a thread
        self.skip_update = False        # used by restart to resubmit jobs as they were rather than doing the next step

    def process(self, running, settings):
        return process.process(self, running, settings)

    def interpret(self, allthreads, running, settings):
        return interpret.interpret(self, allthreads, running, settings)

    def gatekeeper(self, settings):
        jobtype = factory.jobtype_factory(settings.job_type)
        return jobtype.gatekeeper(self, settings)

    def get_next_step(self, settings):
        jobtype = factory.jobtype_factory(settings.job_type)
        self.current_type, self.current_name = jobtype.get_next_step(self, settings)
        return self.current_type, self.current_name

    def get_batch_template(self, type, settings):
        jobtype = factory.jobtype_factory(settings.job_type)
        return jobtype.get_batch_template(self, type, settings)

    def get_frame(self, traj, frame, settings):
        mdengine = factory.mdengine_factory(settings.md_engine)
        return mdengine.get_frame(traj, frame, settings)

    def get_status(self, job_index, settings):
        batchsystem = factory.batchsystem_factory(settings.batch_system)
        return batchsystem.get_status(self.jobids[job_index], settings)

    def cancel_job(self, job_index, settings):
        batchsystem = factory.batchsystem_factory(settings.batch_system)
        batchsystem.cancel_job(self.jobids[job_index], settings)


def init_threads(settings):
    """
    Initialize all the Thread objects called for by the user input file.

    In the case where settings.restart == True, this involves unpickling restart.pkl; otherwise, brand new objects are
    produced in accordance with settings.job_type (aimless_shooting, committor_analysis, equilibrium_path_sampling, or
    isee).

    Parameters
    ----------
    settings : argparse.Namespace
        Settings namespace object

    Returns
    -------
    allthreads : list
        List of all Thread objects, including those for which no further tasks are scheduled.

    """

    if settings.restart:
        allthreads = pickle.load(open(settings.working_directory + '/restart.pkl', 'rb'))
        for thread in allthreads:
            if not thread.current_type == []:
                thread.skip_update = True
        if settings.restart_terminated_threads:
            for thread in allthreads:
                thread.terminated = False
        if settings.job_type == 'aimless_shooting' and settings.information_error_checking:
            if os.path.exists(settings.working_directory + '/info_err.out') and len(open(settings.working_directory + '/info_err.out', 'r').readlines()) > 0:
                info_err_lines = open(settings.working_directory + '/info_err.out', 'r').readlines()

                # Resample completely if there's been a change in the number of definitions of CVs, or in the settings
                # for, information_error_max_dims or information_error_lmax_string
                wrong_length = False
                for data_length in [str(line.split()[0]) for line in info_err_lines]:
                    first_line = open(settings.working_directory + '/as_decorr_' + data_length + '.out', 'r').readlines()[0]
                    num_cvs = len(first_line.replace('A <- ', '').replace('B <- ', '').split())
                    if settings.include_qdot:
                        num_cvs = num_cvs / 2
                    if not num_cvs == len(settings.cvs):
                        wrong_length = True
                if (settings.previous_cvs and not settings.previous_cvs == settings.cvs) or \
                        (not settings.previous_information_error_max_dims == settings.information_error_max_dims) or \
                        (not settings.previous_information_error_lmax_string == settings.information_error_lmax_string) or \
                        wrong_length:
                    utilities.resample(settings, partial=False)
                    information_error.main()

                # Resample if info_err.out is improperly formatted (will not run if resample called above)
                if False in [len(info_err_lines[i].split(' ')) == 2 for i in range(0, len(info_err_lines))]:
                    utilities.resample(settings, partial=True)
                    information_error.main()

                # Resample if info_err.out is missing lines (will not run if resample called above)
                len_data = len(open(settings.working_directory + '/as_raw.out', 'r').readlines())
                last_info_err = info_err_lines[-1].split(' ')[0]
                last_breakpoint = len_data - (len_data % settings.information_error_freq)
                if (last_breakpoint > 0 and not int(last_info_err) == int(last_breakpoint)):
                    utilities.resample(settings, partial=True)
                    information_error.main()
        if settings.job_type == 'equilibrium_path_sampling' and settings.eps_dynamic_seed:    # handle dynamic seeding restart behavior
            for thread in allthreads:
                window_index = 0
                for bounds in settings.eps_bounds:
                    if bounds == thread.history.bounds:
                        settings.eps_empty_windows[window_index] -= 1  # decrement empty window count in this window
                        if settings.eps_empty_windows[window_index] < 0:  # minimum value 0
                            settings.eps_empty_windows[window_index] = 0
                        break
                    window_index += 1

        return allthreads

    # If not restart:
    allthreads = []
    jobtype = factory.jobtype_factory(settings.job_type)

    # Set topology properly even if it's given as a path
    og_prmtop = settings.topology
    if '/' in settings.topology:
        settings.topology = settings.topology[settings.topology.rindex('/') + 1:]
    try:
        shutil.copy(og_prmtop, settings.working_directory + '/' + settings.topology)
    except shutil.SameFileError:
        pass

    for file in jobtype.get_initial_coordinates(settings):
        if '/' in file:
            file = file[file.rindex('/') + 1:]          # drop path to file from filename

        thread = Thread()   # initialize the thread object

        thread.topology = settings.topology

        jobtype.update_history(thread, settings, **{'initialize': True, 'inpcrd': file})    # initialize thread.history

        thread.name = file + '_' + str(thread.suffix)
        allthreads.append(thread)

    return allthreads


def handle_loop_exception(attempted_rescue, running, settings):
    """
    Handle attempted rescue of main loop after encountering an exception, or cancellation of jobs if rescue fails.

    Parameters
    ----------
    attempted_rescue : bool
        True if rescue has already been attempted and this function is being called again. Skips attempting rescue again
        and simply cancels all running jobs.
    running : list
        List of Thread objects that are currently running. These are the threads that will be canceled if the ATESA run
        cannot be rescued.
    settings : argparse.Namespace
        Settings namespace object

    Returns
    -------
    None

    """

    # class UnableToRescueException(Exception):
    #     """ Custom exception for closing out ATESA after an unsuccessful rescue attempt """
    #     pass

    # # todo: finish implementing and then uncomment this
    # if not attempted_rescue:
    #     print('Attempting to remove offending thread(s) and rescue the operation...')
    #     verify_outcome_str, verify_outcome_int = verify_threads.main('restart.pkl')
    #     print(verify_outcome_str)
    #
    #     if verify_outcome_int == 1:  # broken thread removed, continue with attempted rescue
    #         # First, set rescue_running equal to running with deleted threads removed
    #         remaining_threads = pickle.load(open('restart.pkl', 'rb'))
    #         rescue_running = [thread for thread in running if thread in remaining_threads]
    #
    #         # If rescue_running == running, removed threads weren't running so rescue fails. Otherwise...
    #         if not rescue_running == running:
    #             # Then, cancel jobs belonging to deleted threads
    #             deleted_threads = list(set(running) - set(rescue_running))
    #             for thread in deleted_threads:
    #                 try:
    #                     for job_index in range(thread.jobids):
    #                         thread.cancel_job(job_index, settings)
    #                 except Exception as little_e:
    #                     print('Encountered exception while attempting to cancel a job: ' + str(little_e) +
    #                           '\nIgnoring and continuing...')
    #
    #             # Finally, resubmit main() with rescue_running list
    #             main(settings, rescue_running=rescue_running)
    #             return None

    # This code reached if return statement above is not
    print('\nCancelling currently running batch jobs belonging to this process in order to '
          'preserve resources.')
    for thread in running:
        try:
            for job_index in range(len(thread.jobids)):
                thread.cancel_job(job_index, settings)
        except Exception as little_e:
            print('\nEncountered exception while attempting to cancel a job: ' + str(little_e) +
                  '\nIgnoring and continuing...')

    raise RuntimeError('Job cancellation complete, ATESA is now shutting down.')

def main(settings, rescue_running=[]):
    """
    Perform the primary loop of building, submitting, monitoring, and analyzing jobs.

    This function works via a loop of calls to thread.process and thread.interpret for each thread that hasn't
    terminated, until either the global termination criterion is met or all the individual threads have completed.

    Parameters
    ----------
    settings : argparse.Namespace
        Settings namespace object
    rescue_running : list
        List of threads passed in from handle_loop_exception, containing running threads. If given, setup is skipped and
        the function proceeds directly to the main loop.

    Returns
    -------
    exit_message : str
        A message indicating the status of ATESA at the end of main

    """

    if not rescue_running:
        # Implement resample
        if settings.job_type in ['aimless_shooting', 'committor_analysis', 'umbrella_sampling'] and settings.resample:
            # Store settings object in the working directory for compatibility with analysis/utility scripts
            if not settings.dont_dump:
                temp_settings = copy.deepcopy(settings)  # initialize temporary copy of settings to modify
                temp_settings.__dict__.pop('env')  # env attribute is not picklable
                pickle.dump(temp_settings, open(settings.working_directory + '/settings.pkl', 'wb'), protocol=2)
            # Run resampling
            if settings.job_type == 'aimless_shooting':
                utilities.resample(settings, partial=False, full_cvs=settings.full_cvs, only_full_cvs=settings.only_full_cvs)
                if settings.information_error_checking:     # update info_err.out if called for by settings
                    information_error.main()
            elif settings.job_type == 'committor_analysis':
                resampling.resample_committor_analysis(settings)
            elif settings.job_type == 'umbrella_sampling':
                resampling.resample_umbrella_sampling(settings)
            return 'Resampling complete'

        # Make working directory if it does not exist, handling overwrite and restart as needed
        if os.path.exists(settings.working_directory):
            if settings.overwrite and not settings.restart:
                if os.path.exists(settings.working_directory + '/cvs.txt'):     # a kludge to avoid removing cvs.txt
                    if os.path.exists('ATESA_TEMP_CVS.txt'):
                        raise RuntimeError('tried to create temporary file ATESA_TEMP_CVS.txt in directory: ' +
                                           os.getcwd() + ', but it already exists. Please move, delete, or rename it.')
                    shutil.move(settings.working_directory + '/cvs.txt', 'ATESA_TEMP_CVS.txt')
                shutil.rmtree(settings.working_directory)
                os.mkdir(settings.working_directory)
                if os.path.exists('ATESA_TEMP_CVS.txt'):    # continuation of aforementioned kludge
                    shutil.move('ATESA_TEMP_CVS.txt', settings.working_directory + '/cvs.txt')
            elif not settings.restart and glob.glob(settings.working_directory + '/*') == [settings.working_directory + '/cvs.txt']:
                # Occurs when restart = False, overwrite = False, and auto_cvs is used
                pass
            elif not settings.restart:
                raise RuntimeError('Working directory ' + settings.working_directory + ' already exists, but overwrite '
                                   '= False and restart = False. Either change one of these two settings or choose a '
                                   'different working directory.')
        else:
            if not settings.restart:
                os.mkdir(settings.working_directory)
            else:
                raise RuntimeError('Working directory ' + settings.working_directory + ' does not yet exist, but '
                                   'restart = True.')

        # Store settings object in the working directory for compatibility with analysis/utility scripts
        if os.path.exists(settings.working_directory + '/settings.pkl'):    # for checking for need for resample later
            previous_settings = pickle.load(open(settings.working_directory + '/settings.pkl', 'rb'))
            settings.previous_cvs = previous_settings.cvs
            try:
                settings.previous_information_error_max_dims = previous_settings.information_error_max_dims
            except AttributeError:
                pass
            try:
                settings.previous_information_error_lmax_string = previous_settings.information_error_lmax_string
            except AttributeError:
                pass
        if not settings.dont_dump:
            temp_settings = copy.deepcopy(settings)  # initialize temporary copy of settings to modify
            temp_settings.__dict__.pop('env')  # env attribute is not picklable (update: maybe no longer true, but doesn't matter)
            pickle.dump(temp_settings, open(settings.working_directory + '/settings.pkl', 'wb'), protocol=2)

        # Build or load threads
        allthreads = init_threads(settings)

        # Move runtime to working directory
        os.chdir(settings.working_directory)

        running = allthreads.copy()     # to be pruned later by thread.process()
        attempted_rescue = False        # to keep track of general error handling below
    else:
        allthreads = pickle.load(open(settings.working_directory + '/restart.pkl', 'rb'))
        running = rescue_running
        attempted_rescue = True

    # Initialize threads with first process step
    try:
        if not rescue_running:  # if rescue_running, this step has already finished and we just want the while loop
            for thread in allthreads:
                running = thread.process(running, settings)
    except Exception as e:
        if settings.restart:
            print('The following error occurred while attempting to initialize threads from restart.pkl. It may be '
                  'corrupted.')
                  #'If you haven\'t already done so, consider running verify_threads.py to remove corrupted threads from this file.'
        raise e

    try:
        if settings.job_type == 'aimless_shooting' and len(os.sched_getaffinity(0)) > 1:
            # Initialize Manager for shared data across processes; this is necessary because multiprocessing is being
            # retrofitted to code designed for serial processing, but it works!
            manager = Manager()

            # Setup Managed allthreads list
            managed_allthreads = []
            for thread in allthreads:
                thread_dict = thread.__dict__
                thread_history_dict = thread.history.__dict__
                managed_thread = Thread()
                managed_thread.history = manager.Namespace()
                managed_thread.__dict__.update(thread_dict)
                managed_thread.history.__dict__.update(thread_history_dict)
                managed_allthreads.append(managed_thread)
            allthreads = manager.list(managed_allthreads)

            # Setup Managed settings Namespace
            settings_dict = settings.__dict__
            managed_settings = manager.Namespace()
            # Need to explicitly update every key because of how the Managed Namespace works.
            # Calling exec is the best way to do this I could find. Updating managed_settings.__dict__ doesn't work.
            for key in settings_dict.keys():
                exec('managed_settings.' + key + ' = settings_dict[key]')

            # Distribute processes among available core Pool
            with get_context("spawn").Pool(len(os.sched_getaffinity(0))) as p:
                p.starmap(main_loop, zip(itertools.repeat(managed_settings), itertools.repeat(allthreads), [[thread] for thread in allthreads]))
        else:
            main_loop(settings, allthreads, running)
    except AttributeError:  # os.sched_getaffinity raises AttributeError on non-UNIX systems.
        main_loop(settings, allthreads, running)

    ## Deprecated thread pool
    # pool = ThreadPool(len(allthreads))
    # func = partial(main_loop, settings)
    # results = pool.map(func, [[thread] for thread in allthreads])

    jobtype = factory.jobtype_factory(settings.job_type)
    jobtype.cleanup(settings)

    return 'ATESA run exiting normally'

def main_loop(settings, allthreads, running):
    termination_criterion = False
    attempted_rescue = False
    interpreted = []
    # Begin main loop
    # This whole thing is in a try-except block to handle cancellation of jobs when the code crashes in any way
    try:
        while (not termination_criterion) and running:
            for thread in running:
                if thread.gatekeeper(settings):
                    termination_criterion, running = thread.interpret(allthreads, running, settings)
                    if attempted_rescue == True:
                        interpreted.append(thread)
                    if termination_criterion:
                        for thread in running:
                            for job_index in range(len(thread.current_type)):
                                thread.cancel_job(job_index, settings)
                        running = []
                        if not settings.pid == -1:  # finish up currently running resample_and_inferr, if appropriate
                            proc_status = 'running'
                            while proc_status == 'running':
                                try:
                                    proc = psutil.Process(settings.pid).status()
                                    if proc in [psutil.STATUS_RUNNING, psutil.STATUS_SLEEPING, psutil.STATUS_DISK_SLEEP]:
                                        proc_status = 'running'
                                        time.sleep(60)  # wait 1 minute before checking again
                                    else:
                                        proc_status = 'not_running'
                                except (psutil.NoSuchProcess, ProcessLookupError):
                                    proc_status = 'not_running'
                        break
                    running = thread.process(running, settings)
                else:   # todo: change this to only run if all running threads fail gatekeeper?
                    time.sleep(30)  # to prevent too-frequent calls to batch system by thread.gatekeeper

            if all([thread in interpreted for thread in running]):
                attempted_rescue = False    # every thread has passed at an interpret step, so rescue was successful!

    except Exception as e:
        print(str(e))
        handle_loop_exception(attempted_rescue, running, settings)


def run_main():
    # Obtain settings namespace, initialize threads, and move promptly into main.
    try:
        working_directory = sys.argv[2]
    except IndexError:
        working_directory = ''
    try:
        settings = configure.configure(sys.argv[1], working_directory)
    except IndexError:
        raise RuntimeError('No configuration file specified. See documentation at atesa.readthedocs.io for details.')
    exit_message = main(settings)
    print(exit_message)

if __name__ == "__main__":
    run_main()



"""
Unit and regression test for utilities.py.
"""

# Import package, test suite, and other packages as needed
import atesa
import sys
import os
import shutil
import pytest
import pickle
import subprocess
import glob
from atesa import rc_eval
from atesa.configure import configure
from atesa import main

class Tests(object):
    def setup_method(self, test_method):
        try:
            if not os.path.exists('atesa/tests/test_temp'):
                os.mkdir('atesa/tests/test_temp')
            os.chdir('atesa/tests/test_temp')
        except FileNotFoundError:
            pass

    def test_main_pytraj(self):
        """Tests rc_eval.main using sham shooting points in test_data and pytraj cvs"""
        settings = configure('../../data/atesa.config')
        settings.topology = '../test_data/test.prmtop'
        settings.cvs = ['pytraj.distance(traj, \'@1 @2\')[0]', 'pytraj.angle(traj, \'@2 @3 @4\')[0]']
        settings.include_qdot = False
        shutil.copy('../test_data/test_velocities_init.rst7', '../test_temp')
        shutil.copy('../test_data/test_two_init.rst7', '../test_temp')
        shutil.copy('../test_data/as.out', '../test_temp')

        # First test without extant settings.pkl file
        with pytest.raises(FileNotFoundError):
            rc_eval.main('../test_temp/', '3*CV1 - 0.3*CV2', '../test_data/as.out')
        process = subprocess.Popen('rc_eval.py ../test_temp/ 3*CV0-0.3*CV1 ../test_data/as.out', stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True, shell=True)
        output = process.stdout.read().decode()
        assert 'FileNotFoundError' in output    # this section just to ensure it works when called both ways

        settings.__dict__.pop('env')    # env attribute is not picklable
        pickle.dump(settings, open('settings.pkl', 'wb'), protocol=2)   # main will look for this file to load in settings
        # shutil.move('settings.pkl', '../test_data/settings.pkl')
        rc_eval.main('../test_temp/', '3*CV1 - 0.3*CV2', '../test_data/as.out')
        assert os.path.exists('rc.out')
        lines = open('rc.out', 'r').readlines()
        for i in range(len(lines)):     # assert that the rc.out file is properly sorted
            try:
                assert abs(float(lines[i].split(': ')[1])) < abs(float(lines[i+1].split(': ')[1]))
            except IndexError:
                pass

    def test_main_mdtraj(self):
        """Tests rc_eval.main using sham shooting points in test_data and mdtraj cvs"""
        settings = configure('../../data/atesa.config')
        settings.topology = '../test_data/test.prmtop'
        settings.cvs = ['mdtraj.compute_distances(mtraj, numpy.array([[0, 1]]))[0][0] * 10', 'mdtraj.compute_angles(mtraj, numpy.array([[1, 2, 3]]))[0][0] * 180 / numpy.pi']
        settings.include_qdot = False
        shutil.copy('../test_data/test_velocities_init.rst7', '../test_temp')
        shutil.copy('../test_data/test_two_init.rst7', '../test_temp')
        shutil.copy('../test_data/as.out', '../test_temp')

        # First test without extant settings.pkl file
        with pytest.raises(FileNotFoundError):
            rc_eval.main('../test_temp/', '3*CV1 - 0.3*CV2', '../test_data/as.out')
        process = subprocess.Popen('rc_eval.py ../test_temp/ 3*CV0-0.3*CV1 ../test_data/as.out', stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True, shell=True)
        output = process.stdout.read().decode()
        assert 'FileNotFoundError' in output    # this section just to ensure it works when called both ways

        settings.__dict__.pop('env')    # env attribute is not picklable
        pickle.dump(settings, open('settings.pkl', 'wb'), protocol=2)   # main will look for this file to load in settings
        # shutil.move('settings.pkl', '../test_data/settings.pkl')
        rc_eval.main('../test_temp/', '3*CV1 - 0.3*CV2', '../test_data/as.out')
        assert os.path.exists('rc.out')
        lines = open('rc.out', 'r').readlines()
        for i in range(len(lines)):     # assert that the rc.out file is properly sorted
            try:
                assert abs(float(lines[i].split(': ')[1])) < abs(float(lines[i+1].split(': ')[1]))
            except IndexError:
                pass

    def test_extrema_pytraj(self):
        """Tests rc_eval.main with extrema=True using sham shooting points in test_data and pytraj cvs"""
        settings = configure('../../data/atesa.config')
        settings.topology = '../test_data/ethyl_chlorosulfite.prmtop'
        settings.cvs = ['pytraj.distance(traj, \'@1 @7\')[0]', 'pytraj.angle(traj, \'@2 @6 @9\')[0]']
        settings.include_qdot = False
        shutil.copy('../test_data/test_velocities_init.rst7', '../test_temp')
        shutil.copy('../test_data/test_two_init.rst7', '../test_temp')
        shutil.copy('../test_data/as.out', '../test_temp')
        shutil.copy('../test_data/init_-0.20000000000000018_0.rst7_0_us.nc', '../test_temp')
        allthreads = [main.init_threads(settings)[0]]
        allthreads[0].history.prod_trajs = [['init_-0.20000000000000018_0.rst7_0_us.nc', 'init_-0.20000000000000018_0.rst7_0_us.nc']]
        allthreads[0].history.init_coords = ['test_velocities_init.rst7']
        allthreads[0].history.last_accepted = 0
        pickle.dump(allthreads, open('restart.pkl', 'wb'))

        settings.__dict__.pop('env')  # env attribute is not picklable
        pickle.dump(settings, open('settings.pkl', 'wb'),
                    protocol=2)  # main will look for this file to load in settings
        # shutil.move('settings.pkl', '../test_data/settings.pkl')
        extrema = rc_eval.main('../test_temp/', '3*CV1 - 0.3*CV2', '../test_data/as.out', extrema=True)

        assert extrema == [71.4222, 71.4222]

    def test_extrema_mdtraj(self):
        """Tests rc_eval.main with extrema=True using sham shooting points in test_data and pytraj cvs"""
        settings = configure('../../data/atesa.config')
        settings.topology = '../test_data/ethyl_chlorosulfite.prmtop'
        settings.cvs = ['mdtraj.compute_distances(mtraj, numpy.array([[0, 6]]))[0][0] * 10', 'mdtraj.compute_angles(mtraj, numpy.array([[1, 5, 8]]))[0][0] * 180 / numpy.pi']
        settings.include_qdot = False
        shutil.copy('../test_data/test_velocities_init.rst7', '../test_temp')
        shutil.copy('../test_data/test_two_init.rst7', '../test_temp')
        shutil.copy('../test_data/as.out', '../test_temp')
        shutil.copy('../test_data/init_-0.20000000000000018_0.rst7_0_us.nc', '../test_temp')
        allthreads = [main.init_threads(settings)[0]]
        allthreads[0].history.prod_trajs = [['init_-0.20000000000000018_0.rst7_0_us.nc', 'init_-0.20000000000000018_0.rst7_0_us.nc']]
        allthreads[0].history.init_coords = ['test_velocities_init.rst7']
        allthreads[0].history.last_accepted = 0
        pickle.dump(allthreads, open('restart.pkl', 'wb'))

        settings.__dict__.pop('env')  # env attribute is not picklable
        pickle.dump(settings, open('settings.pkl', 'wb'),
                    protocol=2)  # main will look for this file to load in settings
        # shutil.move('settings.pkl', '../test_data/settings.pkl')
        extrema = rc_eval.main('../test_temp/', '3*CV1 - 0.3*CV2', '../test_data/as.out', extrema=True)

        assert extrema == [71.4222, 71.4222]

    @classmethod
    def teardown_method(self, method):
        "Runs at end of class"
        for filename in glob.glob(sys.path[0] + '/atesa/tests/test_temp/*'):
            os.remove(filename)



"""
atesa
Python program for automating the "Aimless Transition Ensemble Sampling and Analysis" (ATESA) aimless shooting workflow on PBS/TORQUE or Slurm.
"""

# Add imports here
#from atesa.main import *
from . import main
from . import factory
from . import configure
from . import mdengine
from . import process
from . import interpret
from . import jobtype
from . import batchsystem
from . import utilities
from . import taskmanager
from . import resampling
# from .configure import *
# from .factory import *
# from .mdengine import *

# Handle versioneer
from ._version import get_versions
versions = get_versions()
__version__ = get_versions()['version']
__git_revision__ = versions['full-revisionid']
del get_versions, versions

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



"""
Unit and regression test for jobtype.py.
"""

# Import package, test suite, and other packages as needed
# import atesa
import pytest
import sys
import glob
import os
import filecmp
import shutil
from atesa.configure import configure
from atesa import factory
from atesa import main

class Tests(object):
    def setup_method(self, test_method):
        try:
            if not os.path.exists('atesa/tests/test_temp'):
                os.mkdir('atesa/tests/test_temp')
            os.chdir('atesa/tests/test_temp')
        except FileNotFoundError:
            pass

    def test_check_termination_aimless_shooting_init(self):
        """Tests check_termination with job_type = 'aimless_shooting' and thread.current_type = ['init']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_termination(allthreads[0], allthreads, settings) == False  # never terminate after an 'init' step

    def test_check_termination_aimless_shooting_prod(self):
        """Tests check_termination with job_type = 'aimless_shooting' and thread.current_type = ['prod', 'prod']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_termination(allthreads[0], allthreads, settings) == False  # todo: update after implementation
        assert allthreads[0].terminated == False              # todo: update after implementation


    def test_update_results_aimless_shooting_init(self):
        """Tests update_results with job_type = 'aimless_shooting' and thread.current_type = ['init']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_results(allthreads[0], allthreads, settings)
        assert os.path.exists('restart.pkl')

    def test_update_results_aimless_shooting_prod(self):
        """Tests update_results with job_type = 'aimless_shooting' and thread.current_type = ['prod', 'prod']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.initial_coordinates = ['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']
        settings.topology = '../test_data/test.prmtop'
        settings.commit_fwd = [[1, 2], [3, 4], [1.5, 2.0], ['lt', 'gt']]
        settings.commit_bwd = [[1, 2], [3, 4], [2.0, 1.5], ['gt', 'lt']]
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_trajs = [['../test_data/test.nc', '../test_data/test.nc']]
        allthreads[0].history.init_coords = [['test_velocities.rst7_0_init.rst7']]
        shutil.copy('../test_data/test_velocities.rst7', 'test_velocities.rst7_0_init.rst7')  # create the needed file
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_results(allthreads[0], allthreads, settings)
        assert os.path.exists('restart.pkl')
        assert os.path.exists('status.txt')
        assert allthreads[0].history.prod_results == [['', '']]

    def test_algorithm_aimless_shooting_init(self):
        """Tests algorithm with job_type = 'aimless_shooting' and thread.current_type = ['init']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        allthreads[0].history.init_coords = [['test_velocities.rst7_0_init.rst7']]
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert allthreads[0].current_type == []     # result for missing .rst7 file (haven't copied it yet)
        allthreads[0].current_type = ['init']       # reset last result
        shutil.copy('../test_data/test_velocities.rst7', 'test_velocities.rst7_0_init.rst7')    # create the needed file
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert allthreads[0].current_type == ['init']   # results for .rst7 was found
        assert allthreads[0].history.init_coords == [['test_velocities.rst7_0_init.rst7', 'test_velocities.rst7_0_init_bwd.rst7']]

    def test_algorithm_aimless_shooting_prod_not_always_new_not_accepted(self):
        """Tests algorithm with job_type = 'aimless_shooting', always_new = False and thread.current_type =
        ['prod', 'prod'] for a move that isn't accepted"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.always_new = False
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_results.append(['fwd', 'fwd'])  # not an accepted move
        allthreads[0].history.prod_trajs.append(['../test_data/test.nc', '../test_data/test.nc'])
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert allthreads[0].history.init_inpcrd[-1] == allthreads[0].history.init_inpcrd[-2]

    def test_algorithm_aimless_shooting_prod_always_new_not_accepted(self):
        """Tests algorithm with job_type = 'aimless_shooting', always_new = True and thread.current_type =
        ['prod', 'prod'] for a move that isn't accepted"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.topology = '../test_data/test.prmtop'
        settings.always_new = True
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        settings.min_dt = -1
        settings.max_dt = -1     # set these to the same value to guarantee which frame is chosen
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_results = [['bwd', 'fwd'], ['fwd', 'fwd']]      # accepted then not accepted
        allthreads[0].history.prod_trajs = [['../test_data/test.nc', '../test_data/test.nc'], ['../test_data/test.nc', '../test_data/test.nc']]
        allthreads[0].suffix = 1
        allthreads[0].history.last_accepted = 0
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert filecmp.cmp(allthreads[0].history.init_inpcrd[1], '../test_data/test.rst7') # test.rst7 is last frame of test.nc
        os.remove('../test_data/test.nc_frame_-1.rst7') # have to do this manually because aimless shooting's mdengine getframe method keeps the '../test_data/' in front of the file name

    def test_algorithm_aimless_shooting_prod_accepted(self):
        """Tests algorithm with job_type = 'aimless_shooting' and thread.current_type = ['prod', 'prod'] for an accepted move"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.topology = '../test_data/test.prmtop'
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        settings.degeneracy = 1
        settings.min_dt = -1
        settings.max_dt = -1     # set these to the same value to guarantee which frame is chosen
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_results = [['bwd', 'bwd'], ['fwd', 'bwd']]  # not accepted then accepted
        allthreads[0].history.prod_trajs = [['not_a_real_file.nc', 'not_a_real_file.nc'], ['../test_data/test.nc', '../test_data/test.nc']]
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert filecmp.cmp(allthreads[0].history.init_inpcrd[1], '../test_data/test.rst7') # test.rst7 is last frame of test.nc
        os.remove('../test_data/test.nc_frame_-1.rst7') # have to do this manually because aimless shooting's mdengine getframe method keeps the '../test_data/' in front of the file name

    def test_update_history_aimless_shooting_init(self):
        """Tests update_history with job_type = 'aimless_shooting' and thread.current_type = ['init']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        these_kwargs = {'rst': 'fakey_mcfakename.rst'}
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_history(allthreads[0], settings, **these_kwargs)
        assert allthreads[0].history.init_coords[-1] == ['fakey_mcfakename.rst']

    def test_update_history_aimless_shooting_prod(self):
        """Tests update_history with job_type = 'aimless_shooting' and thread.current_type = ['prod', 'prod']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        these_kwargs = {'nc': 'fakey_mcfakename.nc'}
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_history(allthreads[0], settings, **these_kwargs)
        assert allthreads[0].history.prod_trajs[-1] == ['fakey_mcfakename.nc']
        jobtype.update_history(allthreads[0], settings, **these_kwargs)
        assert allthreads[0].history.prod_trajs[-1] == ['fakey_mcfakename.nc', 'fakey_mcfakename.nc']

    def test_get_inpcrd_aimless_shooting_init(self):
        """Tests get_inpcrd with job_type = 'aimless_shooting' and thread.current_type = ['init']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_inpcrd(allthreads[0]) == allthreads[0].history.init_inpcrd
        assert allthreads[0].history.init_inpcrd == ['test_velocities.rst7']

    def test_get_inpcrd_aimless_shooting_prod(self):
        """Tests get_inpcrd with job_type = 'aimless_shooting' and thread.current_type = ['prod', 'prod']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.init_coords = [['not_a_real_file_at_all_init.rst7', 'not_a_real_file_at_all_init_bwd.rst7']]
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_inpcrd(allthreads[0]) == ['not_a_real_file_at_all_init.rst7', 'not_a_real_file_at_all_init_bwd.rst7']

    def test_get_initial_coordinates_aimless_shooting(self):
        """Tests get_initial_coordinates with job_type = 'aimless_shooting'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.initial_coordinates = ['../test_data/test.rst7', '../test_data/test_two_init.rst7']
        settings.degeneracy = 2
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_initial_coordinates(settings) == ['test.rst7_0', 'test.rst7_1', 'test_two_init.rst7_0', 'test_two_init.rst7_1']

    def test_gatekeep_aimless_shooting(self):
        """Tests gatekeep with job_type = 'aimless_shooting'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.DEBUG = True
        allthreads = main.init_threads(settings)
        allthreads[0].jobids = ['123456']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.gatekeeper(allthreads[0], settings) == True

    def test_check_for_successful_step_aimless_shooting_init(self):
        """Tests check_for_successful_step with job_type = 'aimless_shooting' and thread.current_type = ['init']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        allthreads[0].history.init_coords = [['some_init_coords.rst7']]
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_for_successful_step(allthreads[0], settings) == False    # necessary file does not yet exist
        shutil.copy('../test_data/test.rst7', 'some_init_coords.rst7')      # make the necessary file
        assert jobtype.check_for_successful_step(allthreads[0], settings) == True     # necessary file exists

    def test_get_next_step_aimless_shooting_init(self):
        """Tests thread.get_next_step with job_type = 'aimless_shooting' and type = ['init']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        assert allthreads[0].get_next_step(settings) == (['prod', 'prod'], ['fwd', 'bwd'])

    def test_get_next_step_aimless_shooting_prod(self):
        """Tests thread.get_next_step with job_type = 'aimless_shooting' and type = ['fwd','bwd']"""
        settings = configure('../../data/atesa.config')
        settings.degeneracy = 1
        settings.job_type = 'aimless_shooting'
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod','prod']
        assert allthreads[0].get_next_step(settings) == (['init'], ['init'])

    def test_check_for_successful_step_aimless_shooting_prod(self):
        """Tests check_for_successful_step with job_type = 'aimless_shooting' and thread.current_type = ['prod', 'prod']"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_trajs = [['some_prod_traj_1.nc', 'some_prod_traj_2.nc']]
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_for_successful_step(allthreads[0], settings) == False    # necessary files do not yet exist
        shutil.copy('../test_data/test.nc', 'some_prod_traj_1.nc')          # make one necessary file
        assert jobtype.check_for_successful_step(allthreads[0], settings) == False    # still missing one
        shutil.copy('../test_data/test.nc', 'some_prod_traj_2.nc')          # make other necessary file
        assert jobtype.check_for_successful_step(allthreads[0], settings) == True     # both files exist

    def test_get_batch_file_aimless_shooting_amber(self):
        """Tests thread.get_batch_template with job_type = 'aimless_shooting' and md_engine = 'amber'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.md_engine = 'amber'
        allthreads = main.init_threads(settings)
        assert allthreads[0].get_batch_template('init', settings) == 'amber_' + settings.batch_system + '.tpl'

    def test_get_batch_file_aimless_shooting_broken(self):
        """Tests thread.get_batch_template with job_type = 'aimless_shooting' and invalid type = 'fwdd'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        allthreads = main.init_threads(settings)
        with pytest.raises(ValueError):
            allthreads[0].get_batch_template('fwdd', settings)

    def test_check_termination_committor_analysis(self):
        """Tests check_termination with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.committor_analysis_use_rc_out = False
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod', 'prod']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_termination(allthreads[0], allthreads, settings) == False
        assert allthreads[0].terminated == True

    def test_update_results_committor_analysis(self):
        """Tests update_results with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.initial_coordinates = ['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']
        settings.committor_analysis_use_rc_out = False
        settings.topology = '../test_data/test.prmtop'
        settings.commit_fwd = [[1, 2], [3, 4], [1.5, 2.0], ['lt', 'gt']]
        settings.commit_bwd = [[1, 2], [3, 4], [2.0, 1.5], ['gt', 'lt']]
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod', 'prod']
        allthreads[0].history.prod_trajs = ['../test_data/test.nc', '../test_data/test.nc', '../test_data/test.nc']
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_results(allthreads[0], allthreads, settings)
        assert os.path.exists('committor_analysis.out')
        assert allthreads[0].history.prod_results == ['', '', '']

    def test_update_history_committor_analysis(self):
        """Tests update_history with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        settings.committor_analysis_use_rc_out = False
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod', 'prod']
        these_kwargs = {'nc': 'fakey_mcfakename.nc'}
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_history(allthreads[0], settings, **these_kwargs)
        assert allthreads[0].history.prod_trajs == ['fakey_mcfakename.nc']
        jobtype.update_history(allthreads[0], settings, **these_kwargs)
        assert allthreads[0].history.prod_trajs == ['fakey_mcfakename.nc', 'fakey_mcfakename.nc']

    def test_get_inpcrd_committor_analysis(self):
        """Tests get_inpcrd with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        settings.committor_analysis_use_rc_out = False
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod', 'prod']
        allthreads[0].history.prod_inpcrd = ['not_a_real_file_at_all_init.rst7']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_inpcrd(allthreads[0]) == ['not_a_real_file_at_all_init.rst7', 'not_a_real_file_at_all_init.rst7', 'not_a_real_file_at_all_init.rst7']

    def test_get_initial_coordinates_committor_analysis_rc_out(self):
        """Tests get_initial_coordinates with job_type = 'committor_analysis' using an RC out file"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.committor_analysis_use_rc_out = True
        settings.path_to_rc_out = '../test_data/rc_eval.out'
        settings.rc_threshold = 0.002
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_initial_coordinates(settings) == ['1.1_1.2_2.2_2.6.rst7_2_10_init_fwd.rst_2_1_2.rst7_5_init_fwd.rst', '1.1_1.2_2.2_2.6.rst7_2_10_init_fwd.rst_4_58_3.rst7_4_init_fwd.rst']

    def test_get_initial_coordinates_committor_analysis_rc_out_does_not_exist(self):
        """Tests get_initial_coordinates with job_type = 'committor_analysis' using an RC out file that does not exist"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.committor_analysis_use_rc_out = True
        settings.path_to_rc_out = '../test_data/rc_eval_FAKE.out'
        settings.rc_threshold = 0.002
        jobtype = factory.jobtype_factory(settings.job_type)
        with pytest.raises(FileNotFoundError):
            allthreads = main.init_threads(settings)

    def test_get_initial_coordinates_committor_analysis_rc_out_no_shooting_points(self):
        """Tests get_initial_coordinates with job_type = 'committor_analysis' using an RC out file containing no
        shooting points within the threshold"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.committor_analysis_use_rc_out = True
        settings.path_to_rc_out = '../test_data/rc_eval.out'
        settings.rc_threshold = 0.0002
        jobtype = factory.jobtype_factory(settings.job_type)
        with pytest.raises(RuntimeError):
            allthreads = main.init_threads(settings)

    def test_gatekeep_committor_analysis(self):
        """Tests gatekeep with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.DEBUG = True
        settings.committor_analysis_use_rc_out = False
        allthreads = main.init_threads(settings)
        allthreads[0].jobids = ['123456']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.gatekeeper(allthreads[0], settings) == True

    def test_check_for_successful_step_committor_analysis(self):
        """Tests check_for_successful_step with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.committor_analysis_use_rc_out = False
        allthreads = main.init_threads(settings)
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_for_successful_step(allthreads[0], settings) == True

    def test_get_next_step_committor_analysis(self):
        """Tests get_next_step with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.committor_analysis_use_rc_out = False
        settings.committor_analysis_n = 3
        allthreads = main.init_threads(settings)
        jobtype = factory.jobtype_factory(settings.job_type)
        this_types, this_names = jobtype.get_next_step(allthreads[0], settings)
        assert this_types == ['prod', 'prod', 'prod']
        assert this_names == ['0', '1', '2']

    def test_get_batch_template_committor_analysis(self):
        """Tests get_batch_template with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.md_engine = 'amber'
        settings.batch_system = 'slurm'
        settings.committor_analysis_use_rc_out = False
        settings.path_to_templates = sys.path[0] + '/atesa/data/templates'
        allthreads = main.init_threads(settings)
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_batch_template(allthreads[0], 'prod', settings) == 'amber_slurm.tpl'

    def test_check_termination_equilibrium_path_sampling_global(self):
        """Tests check_termination global criterion with job_type = 'equilibrium_path_sampling'"""
        settings = config_equilibrium_path_sampling()
        settings.samples_per_window = -1
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_termination(allthreads[0], allthreads, settings) == False
        assert allthreads[0].terminated == False

    def test_check_termination_equilibrium_path_sampling_thread(self):
        """Tests check_termination thread criterion with job_type = 'equilibrium_path_sampling'"""
        settings = config_equilibrium_path_sampling()
        settings.initial_coordinates = ['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_trajs = [['../test_data/test.nc', '../test_data/test.nc']]
        allthreads[0].history.prod_lens = [[2, 2]]
        allthreads[0].history.init_coords = [['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']]
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_results(allthreads[0], allthreads, settings)
        settings.samples_per_window = 1
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_termination(allthreads[0], allthreads, settings) == False  # global termination False
        assert allthreads[0].terminated == True     # thread termination True

    def test_update_results_equilibrium_path_sampling(self):
        """Tests update_results with job_type = 'equilibrium_path_sampling'"""
        settings = config_equilibrium_path_sampling()
        settings.initial_coordinates = ['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_trajs = [['../test_data/test.nc', '../test_data/test.nc']]
        allthreads[0].history.prod_lens = [[2, 2]]
        allthreads[0].history.init_coords = [['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']]
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_results(allthreads[0], allthreads, settings)
        assert os.path.exists('eps.out')
        assert allthreads[0].history.prod_results[-1] == pytest.approx([-34.29, -35.89, -35.17, -35.89, -35.17], 1E-3)

    def test_update_history_equilibrium_path_sampling_prod(self):
        """Tests update_history with job_type = 'equilibrium_path_sampling' with current_type = ['prod', 'prod']"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        these_kwargs = {'nc': 'fakey_mcfakename.nc'}
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_history(allthreads[0], settings, **these_kwargs)
        assert allthreads[0].history.prod_trajs == [['fakey_mcfakename.nc']]
        jobtype.update_history(allthreads[0], settings, **these_kwargs)
        assert allthreads[0].history.prod_trajs == [['fakey_mcfakename.nc', 'fakey_mcfakename.nc']]

    def test_update_history_equilibrium_path_sampling_init(self):
        """Tests update_history with job_type = 'equilibrium_path_sampling' with current_type = ['init']"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        these_kwargs = {'rst': 'fakey_mcfakename.rst7'}
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.update_history(allthreads[0], settings, **these_kwargs)
        assert allthreads[0].history.init_coords == [['fakey_mcfakename.rst7']]

    def test_update_history_equilibrium_path_sampling_out_of_bounds(self):
        """Tests update_history with job_type = 'equilibrium_path_sampling' with an initially out-of-bounds RC value"""
        shutil.copy('../../data/atesa.config', 'eps.config')
        with open('eps.config', 'a') as f:  # need to set these things before calling configure()
            f.write('\njob_type = \'equilibrium_path_sampling\'')
            f.write('\neps_rc_min = -0.1')  # crazy wide range so everything gets included
            f.write('\neps_rc_max = 0.1')
            f.write('\neps_rc_step = 0.1')
            f.write('\neps_overlap = 0.01')
            f.write('\neps_dynamic_seed = 3')
            f.close()
        settings = configure('eps.config')
        settings.DEBUG = True
        settings.job_type = 'equilibrium_path_sampling'
        settings.topology = '../test_data/test.prmtop'
        settings.rc_reduced_cvs = False
        settings.include_qdot = False
        settings.cvs = ['pytraj.distance(traj, \'@1 @2\')[0]', 'pytraj.angle(traj, \'@2 @3 @4\')[0]']
        settings.rc_definition = '1.00 + 2.34*CV1 - 0.67*CV2'
        settings.initial_coordinates = ['../test_data/test.rst7']
        with pytest.raises(RuntimeError):
            allthreads = main.init_threads(settings)

    def test_get_inpcrd_equilibrium_path_sampling(self):
        """Tests get_inpcrd with job_type = 'equilibrium_path_sampling'"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.init_coords = [['not_a_real_file_at_all_init.rst7']]
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_inpcrd(allthreads[0]) == ['not_a_real_file_at_all_init.rst7']

    def test_get_initial_coordinates_equilibrium_path_sampling(self):
        """Tests get_initial_coordinates with job_type = 'equilibrium_path_sampling'"""
        settings = config_equilibrium_path_sampling()
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_initial_coordinates(settings) == ['test.rst7']

    def test_gatekeep_equilibrium_path_sampling(self):
        """Tests gatekeep with job_type = 'committor_analysis'"""
        settings = config_equilibrium_path_sampling()
        settings.DEBUG = True
        allthreads = main.init_threads(settings)
        allthreads[0].jobids = ['123456']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.gatekeeper(allthreads[0], settings) == True

    def test_check_for_successful_step_equilibrium_path_sampling(self):
        """Tests check_for_successful_step with job_type = 'equilibrium_path_sampling'"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        allthreads[0].history.init_coords = [['test_velocities.rst7']]
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.check_for_successful_step(allthreads[0], settings) == False
        shutil.copy('../test_data/test_velocities.rst7', 'test_velocities.rst7')
        assert jobtype.check_for_successful_step(allthreads[0], settings) == True
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_trajs = [['test.nc', 'test.nc']]
        assert jobtype.check_for_successful_step(allthreads[0], settings) == False
        shutil.copy('../test_data/test.nc', 'test.nc')
        assert jobtype.check_for_successful_step(allthreads[0], settings) == True

    def test_get_next_step_equilibrium_path_sampling_first(self):
        """Tests get_next_step with job_type = 'equilibrium_path_sampling' and an empty current_type"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = []
        assert allthreads[0].get_next_step(settings) == (['init'], ['init'])

    def test_get_next_step_equilibrium_path_sampling_init(self):
        """Tests get_next_step with job_type = 'equilibrium_path_sampling' and current_type = ['init']"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        assert allthreads[0].get_next_step(settings) == (['prod', 'prod'], ['fwd', 'bwd'])

    def test_get_next_step_equilibrium_path_sampling_prod(self):
        """Tests get_next_step with job_type = 'equilibrium_path_sampling' and current_type = ['prod', 'prod']"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        assert allthreads[0].get_next_step(settings) == (['init'], ['init'])

    def test_get_batch_file_equilibrium_path_sampling_amber(self):
        """Tests thread.get_batch_template with job_type = 'equilibrium_path_sampling' and md_engine = 'amber'"""
        settings = config_equilibrium_path_sampling()
        settings.job_type = 'equilibrium_path_sampling'
        settings.md_engine = 'amber'
        allthreads = main.init_threads(settings)
        assert allthreads[0].get_batch_template('init', settings) == 'amber_' + settings.batch_system + '.tpl'

    def test_get_batch_file_equilibrium_path_sampling_broken(self):
        """Tests thread.get_batch_template with job_type = 'equilibrium_path_sampling' and invalid type = 'fwdd'"""
        settings = config_equilibrium_path_sampling()
        settings.job_type = 'equilibrium_path_sampling'
        allthreads = main.init_threads(settings)
        with pytest.raises(ValueError):
            allthreads[0].get_batch_template('fwdd', settings)

    def test_algorithm_equilibrium_path_sampling_init(self):
        """Tests algorithm with job_type = 'equilibrium_path_sampling' and thread.current_type = ['init']"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        allthreads[0].history.init_coords = [['test_velocities.rst7_0_init.rst7']]
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert allthreads[0].current_type == []     # result for missing .rst7 file (haven't copied it yet)
        allthreads[0].current_type = ['init']       # reset last result
        shutil.copy('../test_data/test_velocities.rst7', 'test_velocities.rst7_0_init.rst7')    # create the needed file
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert allthreads[0].current_type == ['init']   # results for .rst7 was found
        assert allthreads[0].history.init_coords == [['test_velocities.rst7_0_init.rst7', 'test_velocities.rst7_0_init_bwd.rst7']]

    def test_algorithm_equilibrium_path_sampling_prod_not_accepted(self):
        """Tests algorithm with job_type = 'equilibrium_path_sampling' and thread.current_type = ['prod', 'prod'] for a
        move that isn't accepted"""
        settings = config_equilibrium_path_sampling()
        settings.min_dt = -1
        settings.max_dt = -1     # set these to the same value to guarantee which frame is chosen
        settings.eps_dynamic_seed = 0
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_results = [[-34.12, -35.1, -36], [12, 13, 11]]      # accepted then not accepted
        allthreads[0].history.prod_trajs = [['test.nc', 'test.nc'], ['not_a_real_file.nc', 'not_a_real_file.nc']]
        allthreads[0].history.init_coords = [['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']]
        allthreads[0].suffix = 1
        allthreads[0].history.last_accepted = 0
        shutil.copy('../test_data/test.nc', 'test.nc')
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert filecmp.cmp(allthreads[0].history.init_inpcrd[1], '../test_data/test.rst7') # test.rst7 is last frame of test.nc

    def test_algorithm_equilibrium_path_sampling_prod_not_accepted_no_accepted_yet(self):
        """Tests algorithm with job_type = 'equilibrium_path_sampling' and thread.current_type = ['prod', 'prod'] for a
        move that isn't accepted and with no accepted moves in the thread's history"""
        settings = config_equilibrium_path_sampling()
        settings.min_dt = -1
        settings.max_dt = -1     # set these to the same value to guarantee which frame is chosen
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_results = [[16, 13, 15], [12, 13, 11]]      # accepted then not accepted
        allthreads[0].history.prod_trajs = [['test.nc', 'test.nc'], ['test.nc', 'test.nc']]
        allthreads[0].history.init_coords = [['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']]
        allthreads[0].suffix = 1
        shutil.copy('../test_data/test.nc', 'test.nc')
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert filecmp.cmp(allthreads[0].history.init_inpcrd[1], '../test_data/test.rst7') # test.rst7 is last frame of test.nc

    def test_algorithm_equilibrium_path_sampling_prod_accepted(self):
        """Tests algorithm with job_type = 'equilibrium_path_sampling' and thread.current_type = ['prod', 'prod'] for
        an accepted move"""
        settings = config_equilibrium_path_sampling()
        settings.min_dt = -1
        settings.max_dt = -1     # set these to the same value to guarantee which frame is chosen
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod', 'prod']
        allthreads[0].history.prod_results = [[12, 13, 11], [-34.12, -35.1, -36]]  # not accepted then accepted
        allthreads[0].history.prod_trajs = [['not_a_real_file.nc', 'not_a_real_file.nc'], ['test.nc', 'test.nc']]
        allthreads[0].history.init_coords = [['../test_data/test_velocities.rst7', '../test_data/test_velocities.rst7']]
        shutil.copy('../test_data/test.nc', 'test.nc')
        jobtype = factory.jobtype_factory(settings.job_type)
        jobtype.algorithm(allthreads[0], allthreads, allthreads, settings)
        assert filecmp.cmp(allthreads[0].history.init_inpcrd[1], '../test_data/test.rst7') # test.rst7 is last frame of test.nc

    def test_get_input_file_aimless_shooting(self):
        """Tests get_input_file with job_type = 'aimless_shooting'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'aimless_shooting'
        settings.degeneracy = 1
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_input_file(allthreads[0], 0, settings) == settings.path_to_input_files + '/' + settings.job_type + '_' + allthreads[0].current_type[0] + '_' + settings.md_engine + '.in'

    def test_get_input_file_committor_analysis(self):
        """Tests get_input_file with job_type = 'committor_analysis'"""
        settings = configure('../../data/atesa.config')
        settings.job_type = 'committor_analysis'
        settings.committor_analysis_use_rc_out = False
        settings.initial_coordinates = ['../test_data/test_velocities.rst7']
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_input_file(allthreads[0], 0, settings) == settings.path_to_input_files + '/' + settings.job_type + '_' + allthreads[0].current_type[0] + '_' + settings.md_engine + '.in'

    def test_get_input_file_equilibrium_path_sampling(self):
        """Tests get_input_file with job_type = 'equilibrium_path_sampling'"""
        settings = config_equilibrium_path_sampling()
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['init']
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_input_file(allthreads[0], 0, settings) == settings.path_to_input_files + '/' + settings.job_type + '_' + allthreads[0].current_type[0] + '_' + settings.md_engine + '.in'
        allthreads[0].current_type = ['prod', 'prod']
        new_file = jobtype.get_input_file(allthreads[0], 0, settings)
        assert os.path.exists(new_file)
        assert allthreads[0].history.prod_lens[-1][0] + allthreads[0].history.prod_lens[-1][1] == settings.eps_n_steps
        new_file = jobtype.get_input_file(allthreads[0], 1, settings)
        assert os.path.exists(new_file)

    def test_get_input_file_rxncor_umbrella_sampling(self):
        """Tests get_input_file with job_type = 'umbrella_sampling'"""
        settings = configure('../../data/atesa.config')
        settings.cvs = ['pytraj.distance(traj, \'@1 @2\')[0]', '(mdtraj.compute_distances(mtraj, numpy.array([[7177, 7178]]))[0][0] * 10) - (mdtraj.compute_distances(mtraj, numpy.array([[4272, 7178]]))[0][0] * 10)']
        settings.us_rc_min = -2
        settings.us_rc_max = 2
        settings.us_rc_step = 0.5
        settings.us_degeneracy = 2
        settings.us_implementation = 'amber_rxncor'
        settings.job_type = 'umbrella_sampling'
        settings.as_out_file = '../test_data/as.out'
        allthreads = main.init_threads(settings)
        allthreads[0].current_type = ['prod']
        allthreads[0].history.window = 2.5
        allthreads[0].history.index = 1
        jobtype = factory.jobtype_factory(settings.job_type)
        assert jobtype.get_input_file(allthreads[0], 0, settings) == 'umbrella_sampling_2.5_1.in'
        assert filecmp.cmp('umbrella_sampling_2.5_1.in', '../test_data/umbrella_sampling_2.5_1.in')

    @classmethod
    def teardown_method(self, method):
        "Runs at end of each method"
        for filename in glob.glob(sys.path[0] + '/atesa/tests/test_temp/*'):
            os.remove(filename)

def config_equilibrium_path_sampling():
    """Sets up configuration settings for equilibrium path sampling tests, to be overwritten as needed"""
    shutil.copy('../../data/atesa.config', 'eps.config')
    with open('eps.config', 'a') as f:  # need to set these things before calling configure()
        f.write('\njob_type = \'equilibrium_path_sampling\'')
        f.write('\neps_rc_min = -50')   # crazy wide range so everything gets included
        f.write('\neps_rc_max = 50')
        f.write('\neps_rc_step = 1')
        f.write('\neps_rc_overlap = 0.1')
        f.write('\neps_n_steps = 6')
        f.write('\neps_out_freq = 1')
        f.write('\neps_dynamic_seed = 3')
        f.write('\nsamples_per_window = -1')
        f.close()

    settings = configure('eps.config')
    settings.DEBUG = True
    settings.job_type = 'equilibrium_path_sampling'
    settings.topology = '../test_data/test.prmtop'
    settings.rc_reduced_cvs = False
    settings.include_qdot = False
    settings.cvs = ['pytraj.distance(traj, \'@1 @2\')[0]', 'pytraj.angle(traj, \'@2 @3 @4\')[0]']
    settings.rc_definition = '1.00 + 2.34*CV1 - 0.67*CV2'
    settings.initial_coordinates = ['../test_data/test.rst7']

    return settings
